"""
    This file holds an array of all the ckan-compatible uris.
    Adding new uris to this file is as simple as adding a string to an array.
    Using this pattern should not affect the functioning of the methods in wit_main.py
"""

ckan_opendata_portal_uris = [
        "https://daten.rlp.de",
        "http://www.geodata.gov.gr",
        "https://www.datarefuge.org",
        "http://dados.ufrn.br",
        "https://hri.fi/data/fi",
        "http://bermuda.io",
        "https://data.buenosaires.gob.ar",
        "https://opendata.aragon.es",
        "http://catalog.data.ug",
        "http://data.zagreb.hr",
    ]

# ckan_opendata_portal_uris = [
#         "https://daten.rlp.de/api/3/action/package_search",
#         "http://www.geodata.gov.gr/api/3/action/package_search",
#         "https://www.datarefuge.org/api/3/action/package_search",
#         "http://dados.ufrn.br/api/3/action/package_search",
#         "https://hri.fi/data/fi/api/3/action/package_search",
#         "http://bermuda.io/api/3/action/package_search",
#         "https://data.buenosaires.gob.ar/api/3/action/package_search",
#         "https://opendata.aragon.es/api/3/action/package_search",
#         "https://www.datos.gov.py/api/3/action/package_search",
#         "http://data.zagreb.hr/api/3/action/package_search",
#     ]