
# def proxy_settings_1():
#     ext_proxy = {'http': 'http://proxy.ciss.de:3128', 'https': 'http://proxy.ciss.de:3128'}
#     local_noProxy = {'http': None, 'https': None}
#     return ext_proxy['https']